import json
import numpy as np
from sklearn import svm
from metric_learn import LSML_Supervised

# Return two lists: one contains hexagonal (i.e. grid) id's and the other contains corresponding grid features
def gridFeat(obj):
	hexes = obj['features']
	thres = [75, 150, 500, 150, 200, 80, 200, 160, 10, 4]
	id_list = []
	fea_list = []
	for h in hexes:
		if h['properties']['totalCrashes'] > 0:
			dots = h['dots']
			m = float('-inf')
			for i in range(len(dots)):
				f = dots[i]['properties']['feature']
				if sum(f) > m:
					m = sum(f)
					idx = i
			id_list.append(h['id'])
			fea_list.append(dots[idx]['properties']['feature'])
	return id_list, fea_list

class OneClass:
	def __init__(self):
		self.train = []
		self.test = []
		self.anomaly = []
		self.ml_train = []
		self.M = np.identity(10)	# feature dimension
		self.label = []
		self.ids = []

	def start(self, test, ids):
		# Load training data
		with open('app/data/user_data_1/train.json') as f:
			data = json.load(f)
		self.train = data['normal']
		self.anomaly = data['anomaly']
		self.ml_train = self.train + self.anomaly
		self.label = [0]*len(self.train) + [1]*len(self.anomaly)

		# A custom kernel based on metric learning
		def ml_kernel(X, Y):
			gamma = 0.1	# gamma in RBF
			nx = len(X)
			ny = len(Y)
			result = np.zeros([nx, ny])
			for i in range(nx):
				for j in range(ny):
					result[i, j] = np.dot(np.dot((X[i]-Y[j]), self.M), (X[i]-Y[j]).T)
			return np.exp(-gamma*result)

		self.test = test

		clf = svm.OneClassSVM(nu=0.1, kernel=ml_kernel, gamma=0.1)
		clf.fit(self.train)
		colors = clf.decision_function(test).flatten().tolist()
		colors = map(lambda x: 0.1 if x>0 else -x, colors)
		print colors
		ma = max(colors)
		mi = min(colors)
		colors = map(lambda x: (x-mi)/(ma-mi), colors)

		self.ids = ids
		cmap = []
		for i in range(len(ids)):
			c = {}
			c['id'] = ids[i]
			c['color'] = colors[i]
			cmap.append(c)
		return cmap
		
	def update(self, gids, rids):
		for i in range(len(rids)):
			idx = self.ids.index(gids[i])
			self.ml_train.append(self.test[idx])
			if rids[i] == 0:
				self.label.append(0)
				for i in range(20):
					self.train.append(self.test[idx])
			else:
				self.label.append(1)

		lsml = LSML_Supervised(max_iter=100)
		lsml.fit(np.array(self.ml_train), np.array(self.label))
		self.M = lsml.metric()

		def ml_kernel(X, Y):
			gamma = 0.1	# gamma in RBF
			nx = len(X)
			ny = len(Y)
			result = np.zeros([nx, ny])
			for i in range(nx):
				for j in range(ny):
					result[i, j] = np.dot(np.dot((X[i]-Y[j]), self.M), (X[i]-Y[j]).T)
			return np.exp(-gamma*result)

		clf = svm.OneClassSVM(nu=0.1, kernel=ml_kernel, gamma=0.1)
		clf.fit(self.train)
		colors = clf.decision_function(self.test).flatten().tolist()
		colors = map(lambda x: 0.1 if x>0 else -x, colors)
		ma = max(colors)
		mi = min(colors)
		colors = map(lambda x: (x-mi)/(ma-mi), colors)

		cmap = []
		for i in range(len(self.ids)):
			c = {}
			c['id'] = self.ids[i]
			c['color'] = colors[i]
			cmap.append(c)
		return cmap
