# -*- coding: utf-8 -*-
"""
Created on Mon Jul 24 10:07:12 2017

@author: FH
"""

import json
import numpy as np
from random import sample
from scipy import stats, spatial
from numpy import random, mean

# Distribution parameters
dim = 5
mu = np.ones(dim)
Sigma = np.identity(dim)* .25

mu_a = np.linspace(0.9, 1.1, num=dim)
Sigma_a = np.identity(dim) 
         
# Generate features from a multivariate-normal distribution N(mu, Sigma)

def genFeats(mu, Sigma, n):
    feats = random.multivariate_normal(mu, Sigma, n)
    # afeats = random.multivariate_normal(mu_a, Sigma_a, n) # anomaly from another distribution
    normal = []
    anomaly = []
    for i in range(n):
        if isAnomaly(feats[i], mu, Sigma, 0.002):
            anomaly.append(list(feats[i]))
            # anomaly.append(list(afeats[i]))
        if 1-isAnomaly(feats[i], mu, Sigma,0.002):
            normal.append(list(feats[i]))
    print 'Anomaly rate: %s' % (float(len(anomaly))/n)
    return normal, anomaly

# Determine whether feature x is anomaly
def isAnomaly(x, mu, Sigma, eps):
    dist = stats.multivariate_normal(mu, Sigma)
    return dist.pdf(x) < eps


"""
# Detect anomaly based on Mahalanobis distance
def isAnomaly(x, mu, Sigma, thres):
    return spatial.distance.mahalanobis(x, mu, Sigma) > thres    # HACK: Sigma identity, no inverse computed!
"""

# Compute grid feature based on dots inside (feature of the dot that mostly deviates from the mean feature)
def gridFeat(dots):
    feats = list(map(lambda x: x['feature'], dots))
    maxDistance = 0.
    idx = 0
    if len(feats) > 0:
        # centriod = mean(feats, axis=0)
        centriod = mu
        for i in range(len(dots)):
            if maxDistance < spatial.distance.euclidean(dots[i]['feature'], centriod):
                maxDistance = spatial.distance.euclidean(dots[i]['feature'], centriod)
                idx = i
        return dots[idx]['feature']
    else:
        return mu.tolist()

"""
def genData(res, dataNum, aRatio, tech):    
# res: resolution (10/20/30)
# dataNum: approx. average number of dots in each grid (10/20/30)
# aRatio: ratio of anomalies (1%/5%/10%)
# tech: id of technology (0/1)
    #anomalyGrids = sample(range(res**2), 5) # randomly assign 5 grids to be anormalous
    normal, anomaly = genFeats(mu, Sigma, res**2*dataNum)
    grids = []
    #colors = []
    for i in range(res**2):
        grid = {}
        grid['id'] = i
        grid['q'] = random.random()
        grid['pos'] = [i/res, i%res]
        grid['dots'] = []
        grids.append(grid)
    feats = []
    anomalyFlags = []
    for i in range(res**2*dataNum):
        flag = random.random()
        if flag < aRatio:   # sample from anomalies
            feats.append(sample(anomaly, 1)[0])
            anomalyFlags.append(1)
        else:   # sample from normal data
            feats.append(sample(normal, 1)[0])
            anomalyFlags.append(0)
    zscores = stats.mstats.zscore(np.array(feats), axis=0, ddof=1).tolist()
    for i in range(res**2*dataNum):
        dot = {}
        dot['x'] = random.uniform(0.05, 0.95)
        dot['y'] = random.uniform(0.05, 0.95)
        dot['feature'] = feats[i]
        dot['z'] = zscores[i]
        dot['isAnomaly'] = anomalyFlags[i]
        gid = sample(range(res**2), 1)[0]   # randomly allocate to one grid
        grids[gid]['dots'].append(dot)
    for g in grids:
        #p = sum(map(lambda x: x['isAnomaly'], g['dots'])) + random.normal(0, 0.3, 1)[0]
        #colors.append(p)
        g['gridFeat'] = gridFeat(g['dots'])
        g['isAnomaly'] = 1 if sum(map(lambda x: x['isAnomaly'], g['dots'])) > 0 else 0
        for d in g['dots']:
            del d['feature']
    #colors = np.array(colors)
    #colors = (colors-min(colors))/(max(colors)-min(colors))
    #colors /= sum(colors)
    #for i in range(res**2):
    #    grids[i]['p'] = colors[i]
    #map(lambda x: (x['p']-min(colors))/(sum(colors)-res**2*min(colors)), grids)

    data = {}
    data['grids'] = grids
    data['param'] = [res, dataNum, aRatio, tech]
    data['maxZ'] = np.max(zscores)
    data['minZ'] = np.min(zscores)
    return data

# Update Aug 6: aRatio: #anomaly points/#points --> #anomaly grids/#grids
def genData(res, dataNum, aRatio, tech):    
# res: resolution (10/20/30)
# dataNum: approx. average number of dots in each grid (10/20/30)
# aRatio: ratio of anomaly grids (1%/5%/10%)
# tech: id of technology (0/1)
    anomalyGrids = sample(range(res**2), int(res**2*aRatio))
    normal, anomaly = genFeats(mu, Sigma, res**2*dataNum)
    grids = []
    for i in range(res**2):
        grid = {}
        grid['id'] = i
        grid['q'] = random.random()
        grid['pos'] = [i/res, i%res]
        grid['dots'] = []
        grids.append(grid)
    feats = []
    afeats = []
    for i in range(res**2*dataNum):
        feats.append(sample(normal, 1)[0])
    for i in range(len(anomalyGrids)*3):
        afeats.append(sample(anomaly, 1)[0])
    zscores = stats.mstats.zscore(np.array(feats+afeats), axis=0, ddof=1).tolist()
    f = feats + afeats
    for i in range(res**2*dataNum+len(anomalyGrids)*3):
        dot = {}
        dot['x'] = random.uniform(0.05, 0.95)
        dot['y'] = random.uniform(0.05, 0.95)
        dot['feature'] = f[i]
        dot['z'] = zscores[i]
        dot['isAnomaly'] = 0 if i < res**2*dataNum else 1
        gid = sample(range(res**2), 1)[0] if i < res**2*dataNum else sample(anomalyGrids, 1)[0]
        grids[gid]['dots'].append(dot)
    for g in grids:
        g['gridFeat'] = gridFeat(g['dots'])
        g['isAnomaly'] = 1 if sum(map(lambda x: x['isAnomaly'], g['dots'])) > 0 else 0
        for d in g['dots']:
            del d['feature']

    data = {}
    data['grids'] = grids
    data['param'] = [res, dataNum, aRatio, tech]
    data['maxZ'] = np.max(zscores)
    data['minZ'] = np.min(zscores)
    return data
"""

# Update Aug 7: aRatio --> aNum
def genData(res, dataNum, aNum, tech):    
# res: resolution (10/20/30)
# dataNum: approx. average number of dots in each grid (10/20/30)
# aNum: number of anomalies (3/5/7)
# tech: id of technology (0/1)
    anomalyGrids = sample(range(res**2), aNum)
    normal, anomaly = genFeats(mu, Sigma, res**2*dataNum)
    grids = []
    for i in range(res**2):
        grid = {}
        grid['id'] = i
        grid['q'] = random.random()
        grid['pos'] = [i/res, i%res]
        grid['dots'] = []
        grids.append(grid)
    for i in range(res**2*dataNum):
        dot = {}
        dot['x'] = random.uniform(0.05, 0.95)
        dot['y'] = random.uniform(0.05, 0.95)
        dot['feature'] = sample(normal, 1)[0]
        dot['isAnomaly'] = 0
        gid = sample(range(res**2), 1)[0]
        grids[gid]['dots'].append(dot)
    for i in anomalyGrids:
        dot = {}
        dot['x'] = random.uniform(0.05, 0.95)
        dot['y'] = random.uniform(0.05, 0.95)
        dot['feature'] = sample(anomaly, 1)[0]
        dot['isAnomaly'] = 1
        grids[i]['dots'].append(dot)
    for g in grids:
        g['gridFeat'] = gridFeat(g['dots'])
        g['isAnomaly'] = 1 if sum(map(lambda x: x['isAnomaly'], g['dots'])) > 0 else 0
        for d in g['dots']:
            del d['feature']

    data = {}
    data['grids'] = grids
    data['param'] = [res, dataNum, aNum, tech]
    return data
    
def checkGrids(data):
    n = len(data['grids'])
    aCount = []
    dCount = []
    for i in range(n):
        dots = data['grids'][i]['dots']
        aCount.append(sum(map(lambda x: x['isAnomaly'], dots)))
        dCount.append(len(dots))
    return aCount, dCount
    
def writeJson(rep=3):
    res = [10, 20]
    dataNum = [10, 20]
    aNum = [3, 5, 7]
    tech = [0]
    total = len(res)*len(dataNum)*len(aNum)*len(tech)*rep
    i = 1
    output = []
    for j in range(rep):
        for t in tech:
            data = []
            for r in res:
                for d in dataNum:
                    for a in aNum:
                        data.append(genData(r, d, a, t))
                        print 'Rep %s: Task %s-%s-%s-%s is generated (%s/%s)' % (j+1, r, d, a, t, i, total)
                        i += 1
            #random.shuffle(data)
            print '---===Tasks shuffled===---'
            output += data
    i = 1
    for d in output:
        d['name'] = '%s/%s' % (i, total)
        i += 1
    with open('./data.json', 'w') as f:
        json.dump(output, f)
    print '---===Done===---'
    return output
                
if __name__ == '__main__':
	o = writeJson(1)
