var mapView = vis.mapView().size([$("#mainview").width(), $("#mainview").height()]);
var radarview = vis.radarview().size([$("#hview2").width(), $("#hview2").height()]);
// var triangle = vis.triangle().size([$("#hview2").width(), $("#hview2").height()]);
var radar = vis.radar();
// var gzstar = vis.gzstar();

var cellSize = 100; // set the grid size

var flag = 0;

$(document).ready(function() { 
    $.get("/list", function(d) {
        $("#dataset").empty();
        d = $.parseJSON(d);
        d.forEach(function(name) {
            $("#dataset").append(
                "<option>" + name + "</option>"
            );
        });

        mapView.container("mainview");

        radarview.container(d3.select("#hview2").append("svg"));
        radarview.glyph(radar).layout().render();
        // radarview.glyph(gzstar).layout().render();
        // triangle.container(d3.select("#hview2").append("svg"));
        // triangle.glyph(radar).layout().render();

        display();
    });

    // $("#tabs").tabs();
    $("#tabselect").tabs();
    $("#tablists").tabs();

    $("#filter").slider({
        min: 20,
        max: 80,
        slide: function(event, ui) {

            range1 = ui.value / 100.0;
            mapView.setfilter(range1).update();
        }
    });

    wire_events();
});

function wire_events() {
    mapView.dispatch.on('mouseover', function(d) {
        mapView.highlight(d, true);

        radarview.clear();
        radarview.nodes(d).layout().render();
        // triangle.clear();
        // triangle.nodes(d).layout().update();
    });

    mapView.dispatch.on('mouseout', function(d) {
        mapView.highlight(d, false);
    });

    mapView.dispatch.on('select', function(d) {
        if (d.click > 0) {
            mapView.select(d);
            mapView.border();
        }

    });

    mapView.dispatch.on('contextmenu', function(d, grids) {

        d3.event.preventDefault();
        if (grids[1].length > 0 || grids[2].length > 0) {
            var ids = getids(grids[1]).concat(getids(grids[2]));
            var res1 = new Array(grids[1].length).fill(1);
            var res2 = new Array(grids[2].length).fill(0);
            var res = res1.concat(res2);

            console.log(ids);
            console.log(res);
            update(d, ids, res);

            if(d.alg < 2) {
                runner.add();
            }
        }

    });

    // radarview.dispatch.on('option', function(d) {
    //     radar.options(d);
    // });
  
};

function display() {
    d3.selectAll(".region").remove();

    var data = $('#dataset').val();
    if (!data || data == '') {
        return;
    }

    var url = "data/" + data;
    d3.json(url, function(error, d) {
        if(error) {
            console.log(error);
        }
        flag = d.update;
        console.log(d);

        var bbox = getBounds(d);
        console.log(bbox);
        // our cell size, try changing this to a different value like 10
        
        // units of our cells
        var units = 'kilometers';

        // create our hex grid
        var hexGrid = turf.hexGrid(bbox, cellSize, units);
        // console.log(hexGrid);
        // count the number of crashes in each hex grid, this will take a few seconds.
        // in an real life app we would want to display a loading GIF or message here
        console.log('binning data, hold on tight...');
        var hexCrashes = turf.count(hexGrid, d, 'totalCrashes');
        // console.log(hexCrashes);

        var hexset = []
        for (var i = 0; i < hexGrid.features.length; i++) {
            if (hexGrid.features[i].properties['totalCrashes']) {
                var searchWithin = {
                    "type": "FeatureCollection",
                    "features": [{
                        "type": "Feature",
                        "properties": {},
                        "geometry": {
                            "type": "Polygon",
                            "coordinates": hexGrid.features[i].geometry.coordinates
                        }
                    }]
                };

                var ptsWithin = turf.within(d, searchWithin);
                // console.log(ptsWithin)
                // hexset.push(ptsWithin);
                hexCrashes.features[i].dots = ptsWithin.features;
            }
            hexCrashes.features[i].id = i;
        }
        
        console.log(hexCrashes);
        var max = 0;
        var min = 1;
        hexCrashes.features.forEach(function(ele, i) {
            if(ele.properties.totalCrashes > max) {
                max = ele.properties.totalCrashes;
            }

            if(ele.properties.totalCrashes < min) {
                min = ele.properties.totalCrashes;
            }
        });

        hexCrashes.features.forEach(function(ele, i) {
            if(ele.properties.totalCrashes > 0) {
                ele.q = (ele.properties.totalCrashes - max) / (min - max);
            }
        });
        var url = "data/start?input=" + JSON.stringify(hexCrashes);
        //console.log(url);
        d3.json(url, function(error, pValues) {
            if (error) {
                console.log(error)
                return;
            }
            console.log(pValues);

            pValues.forEach(function(element, i) {
                // statements
                hexCrashes.features[element.id].ratio = element.color;
            });


            // show map
            mapView.zoom(3.5).center([104, 37]);
            mapView.regions(hexCrashes).layout().render();

        })

    });

    


};

function getBounds(d) {
    var b = d3.geo.bounds(d);
    // var b = d3.geoBounds(d);
    // return L.bounds(this.project([b[0][0], b[1][1]]), this.project([b[1][0], b[0][1]]));
    return [b[0][0], b[0][1], b[1][0], b[1][1]];
};

function update(d, ids, res) {
    
    if(flag == 0) {
        var url = "data/update?gid=" + ids + "&res=" + res;
        d3.json(url, function(error, pValues) {
            if (error) {
                console.log(error)
                return;
            }

            console.log(pValues);
            mapView.SVMUpdate(pValues);
        });
    } else if(flag == 1) {
         mapView.BayesUpdate(d);
    } else if(flag == 2) {
        return;
    }
    
};

function getids(grids) {
    var ids = [];

    for (var i = 0; i < grids.length; ++i) {
        ids.push(grids[i].id);
    }

    return ids;
};

// indices for features: 0, 1, 2, 4, 6, 8
// name: PM2.5, PM10, SO2, NO2, O3, CO
